<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_redirect_uri_wildcard\Validator;

/**
 * Validates redirect URIs with wildcard support in subdomains.
 */
class RedirectUriValidator implements RedirectUriValidatorInterface {

  /**
   * {@inheritdoc}
   */
  public function validateSchema(string $uri): bool {
    // Check if the URI is valid by trying to parse it.
    $parts = parse_url($uri);
    if ($parts === FALSE || !isset($parts['host'])) {
      return FALSE;
    }

    // Check for invalid port wildcard.
    if (isset($parts['port']) && strpos((string) $parts['port'], '*') !== FALSE) {
      return FALSE;
    }

    // Check protocol - no wildcards allowed.
    if (!isset($parts['scheme']) || strpos($parts['scheme'], '*') !== FALSE) {
      return FALSE;
    }

    // Check if localhost.
    if ($parts['host'] === 'localhost') {
      return $this->validateWithOriginalValidation($uri);
    }

    // Split the host into parts.
    $hostParts = explode('.', $parts['host']);
    if (count($hostParts) < 2) {
      return FALSE;
    }

    // Check if the wildcard is only in the subdomain part.
    $wildcardFound = FALSE;
    foreach ($hostParts as $index => $part) {
      if (strpos($part, '*') !== FALSE) {
        // If we've already found a wildcard or this is the last part
        // or second to last part, it's invalid.
        if ($wildcardFound || $index >= count($hostParts) - 2) {
          return FALSE;
        }
        $wildcardFound = TRUE;
      }
    }

    return $this->validateWithOriginalValidation($uri);
  }

  /**
   * {@inheritdoc}
   */
  public function uriMatchesSchema(string $uri, string $schema): bool {
    // If schema doesn't contain any wildcards, do exact match.
    if (strpos($schema, '*') === FALSE) {
      return $uri === $schema;
    }

    // Validate both URIs first.
    if (!$this->validateSchema($schema)) {
      return FALSE;
    }

    $uriParts = parse_url($uri);
    if ($uriParts === FALSE || !isset($uriParts['scheme']) || !isset($uriParts['host'])) {
      return FALSE;
    }

    // Convert schema to regex pattern.
    $pattern = preg_quote($schema, '/');
    // Handle subdomain wildcard.
    $pattern = str_replace('\\*', '[a-z0-9-_]+', $pattern);
    $pattern = '/^' . $pattern . '$/i';

    return (bool) preg_match($pattern, $uri);
  }

  /**
   * Validates the URI with the original validation.
   *
   * The regex is taken from the simple_oauth module.
   *
   * @param string $uri
   *   The URI to validate.
   *
   * @return bool
   *   TRUE if the URI is valid, FALSE otherwise.
   */
  protected function validateWithOriginalValidation(string $uri) {
    // Make sure the regex is happy by replacing the wildcard
    // with a valid character.
    $uri = str_replace('*', 'mysubdomain', $uri);

    return (bool) preg_match("
        /^
        [a-z][a-z0-9+\-\.]*:\/\/                                # Supporting various URL schemes
        (?:
          (?:[a-z0-9\-\.]|%[0-9a-f]{2})+                        # A domain name or a IPv4 address
          |(?:\[(?:[0-9a-f]{0,4}:)*(?:[0-9a-f]{0,4})\])         # or a well formed IPv6 address
        )
        (?::[0-9]+)?                                            # Server port number (optional)
        (?:[\/|\?]
          (?:[\w#!:\.\?\+=&@$'~*,;\/\(\)\[\]\-]|%[0-9a-f]{2})   # The path and query (optional)
        *)?
      $/xi", $uri
    );
  }

}
