<?php

declare(strict_types=1);

namespace Drupal\Tests\simple_oauth_redirect_uri_wildcard\Unit\Validator;

use Drupal\simple_oauth_redirect_uri_wildcard\Validator\RedirectUriValidator;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\simple_oauth_redirect_uri_wildcard\Validator\RedirectUriValidator
 * @group simple_oauth_redirect_uri_wildcard
 */
class RedirectUriValidatorTest extends UnitTestCase {

  /**
   * The redirect URI validator.
   */
  protected RedirectUriValidator $validator;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->validator = new RedirectUriValidator();
  }

  /**
   * Tests the schema validation.
   *
   * @covers ::validateSchema
   * @dataProvider schemaValidationProvider
   */
  public function testSchemaValidation(string $uri, bool $expected): void {
    $result = $this->validator->validateSchema($uri);
    $this->assertSame($expected, $result);
  }

  /**
   * Data provider for testSchemaValidation.
   */
  public function schemaValidationProvider(): array {
    return [
      'valid uri without wildcard' => [
        'https://example.com',
        TRUE,
      ],
      'valid localhost without port' => [
        'http://localhost',
        TRUE,
      ],
      'valid localhost with port' => [
        'http://localhost:3000',
        TRUE,
      ],
      'valid localhost with path' => [
        'http://localhost/callback',
        TRUE,
      ],
      'valid localhost with port and path' => [
        'http://localhost:3000/auth/callback',
        TRUE,
      ],
      'valid uri with port' => [
        'https://example.com:8443',
        TRUE,
      ],
      'valid uri with subdomain wildcard' => [
        'https://*.example.com',
        TRUE,
      ],
      'valid uri with subdomain wildcard and port' => [
        'https://*.example.com:8443',
        TRUE,
      ],
      'valid uri with multiple subdomains and wildcard' => [
        'https://*.test.example.com',
        TRUE,
      ],
      'valid partial wildcard' => [
        'https://test*.example.com',
        TRUE,
      ],
      'valid vercel deployment schema' => [
        'https://*.vercel.app',
        TRUE,
      ],
      'valid vercel project preview schema' => [
        'https://myproject-git-*-username-team.vercel.app',
        TRUE,
      ],
      'invalid uri' => [
        'not-a-uri',
        FALSE,
      ],
      'invalid wildcard in domain' => [
        'https://example.*.com',
        FALSE,
      ],
      'invalid wildcard in tld' => [
        'https://example.com.*',
        FALSE,
      ],
      'invalid multiple wildcards in host' => [
        'https://*.*.example.com',
        FALSE,
      ],
      'invalid uri without protocol' => [
        '*.example.com',
        FALSE,
      ],
      'invalid wildcard in port' => [
        'https://example.com:*',
        FALSE,
      ],
      'invalid wildcard in protocol' => [
        '*://example.com',
        FALSE,
      ],
      'invalid partial wildcard in protocol' => [
        'http*://example.com',
        FALSE,
      ],
      'invalid protocol with wildcard and subdomain' => [
        '*://*.example.com',
        FALSE,
      ],
    ];
  }

  /**
   * Tests the URI matching against schema.
   *
   * @covers ::uriMatchesSchema
   * @dataProvider uriMatchingProvider
   */
  public function testUriMatching(string $uri, string $schema, bool $expected): void {
    $result = $this->validator->uriMatchesSchema($uri, $schema);
    $this->assertSame($expected, $result);
  }

  /**
   * Data provider for testUriMatching.
   */
  public function uriMatchingProvider(): array {
    return [
      'exact match without wildcard' => [
        'https://example.com',
        'https://example.com',
        TRUE,
      ],
      'exact localhost match' => [
        'http://localhost',
        'http://localhost',
        TRUE,
      ],
      'exact localhost match with port' => [
        'http://localhost:3000',
        'http://localhost:3000',
        TRUE,
      ],
      'exact localhost match with path' => [
        'http://localhost/callback',
        'http://localhost/callback',
        TRUE,
      ],
      'exact localhost match with port and path' => [
        'http://localhost:3000/auth/callback',
        'http://localhost:3000/auth/callback',
        TRUE,
      ],
      'no localhost match with different port' => [
        'http://localhost:3001',
        'http://localhost:3000',
        FALSE,
      ],
      'exact match with port' => [
        'https://example.com:8443',
        'https://example.com:8443',
        TRUE,
      ],
      'no match without wildcard' => [
        'https://test.example.com',
        'https://example.com',
        FALSE,
      ],
      'valid match with wildcard' => [
        'https://test.example.com',
        'https://*.example.com',
        TRUE,
      ],
      'valid match with wildcard and port' => [
        'https://test.example.com:8443',
        'https://*.example.com:8443',
        TRUE,
      ],
      'valid match with hyphen' => [
        'https://my-site.example.com',
        'https://*.example.com',
        TRUE,
      ],
      'valid match with numbers' => [
        'https://site123.example.com',
        'https://*.example.com',
        TRUE,
      ],
      'valid match with underscore' => [
        'https://my_site.example.com',
        'https://*.example.com',
        TRUE,
      ],
      'valid vercel deployment id match' => [
        'https://abcd1234.vercel.app',
        'https://*.vercel.app',
        TRUE,
      ],
      'valid vercel project branch preview match' => [
        'https://myproject-git-feature123-username-team.vercel.app',
        'https://myproject-git-*-username-team.vercel.app',
        TRUE,
      ],
      'valid vercel project branch with slashes' => [
        'https://myproject-git-feature-123-username-team.vercel.app',
        'https://myproject-git-*-username-team.vercel.app',
        TRUE,
      ],
      'no match with invalid characters' => [
        'https://my@site.example.com',
        'https://*.example.com',
        FALSE,
      ],
      'no match with different domain' => [
        'https://test.different.com',
        'https://*.example.com',
        FALSE,
      ],
      'no match with invalid schema' => [
        'https://test.example.com',
        'https://example.*.com',
        FALSE,
      ],
      'no match with invalid uri' => [
        'not-a-uri',
        'https://*.example.com',
        FALSE,
      ],
      'no match with different port' => [
        'https://test.example.com:8080',
        'https://*.example.com:8443',
        FALSE,
      ],
      'no match with wrong protocol' => [
        'http://test.example.com',
        'https://*.example.com',
        FALSE,
      ],
      'no match with wrong vercel project' => [
        'https://otherproject-git-feature123-username-team.vercel.app',
        'https://myproject-git-*-username-team.vercel.app',
        FALSE,
      ],
      'no match with wrong vercel username' => [
        'https://myproject-git-feature123-otheruser-team.vercel.app',
        'https://myproject-git-*-username-team.vercel.app',
        FALSE,
      ],
    ];
  }

}
